             .title        "VRC2 + VRC4 Reader Plugin"


;01/14/01 
;Written by KH
;Version 1.0

;7/26/07
;BootGod
;Thru user variables, supports all VRC2 & 4 variants. Valid values are: 2a,2b,4a,4b,4c,4d,4e

;1/24/09
;BootGod
;Added write to 9004h (vrc4d prg swap control) to ensure 8000 is mapped to 8000 and not C000
             
             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch

temp1:       .equ 00e0h
temp1_lo:    .equ 00e0h
temp1_hi:    .equ 00e1h
temp2:       .equ 00e2h
temp2_lo:    .equ 00e2h
temp2_hi:    .equ 00e3h
temp3:       .equ 00e4h
temp3_lo:    .equ 00e4h
temp3_hi:    .equ 00e5h
temp4:       .equ 00e6h
romsiz:      .equ 00e7h
curr_bank:   .equ 00e8h

cb0_1:        .equ 00eah
cb1_0:        .equ 00ebh
cb1_1:        .equ 00ech

uv1enabled:   .equ 07ceh
vrc_version:  .equ 07cfh

;plugin header defines
p_ver: = 3	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "Konami VRC2 / VRC4 carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "Kevin Horton & BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 32 >> 4,512 >> 4
             .db 32 >> 3,256 >> 3
             .db 8,0

             ;plugin version and flags
             .db p_ver
             .db p_read | p_var1

;plugin code start here

             lda #000h
             sta 09002h

             lda #4
             jsr send_byte

;vrc version and variant must be defined, otherwise exit

             lda uv1enabled
             cmp #0ffh
             beq badvar
             jmp goodvar

badvar:      jmp done
             
goodvar:     lda vrc_version
             cmp #02ah
             bne check4b
             beq vrc2a4b
             
check4b:     cmp #04bh
             bne not_vrc2a

vrc2a4b:     lda #2
             sta cb0_1
             lda #1
             sta cb1_0
             lda #3
             sta cb1_1
             jmp vrc_ready

not_vrc2a:   lda vrc_version
             cmp #02bh
             bne not_vrc2b
             lda #1
             sta cb0_1
             lda #2
             sta cb1_0
             lda #3
             sta cb1_1
             jmp vrc_ready

not_vrc2b:   lda vrc_version
             cmp #04ah
             bne not_vrc4a
             lda #2
             sta cb0_1
             lda #4
             sta cb1_0
             lda #6
             sta cb1_1
             jmp vrc_ready

not_vrc4a:   lda vrc_version
             cmp #04ch
             bne not_vrc4c
             lda #40h
             sta cb0_1
             lda #80h
             sta cb1_0
             lda #0c0h
             sta cb1_1
             jmp vrc_ready

not_vrc4c:   lda vrc_version
             cmp #04dh
             bne not_vrc4d
             lda #8
             sta cb0_1
             lda #4
             sta cb1_0
             lda #0ch
             sta cb1_1
             lda #0
             sta 09004h
             jmp vrc_ready

not_vrc4d:   lda vrc_version
             cmp #04eh
             bne not_vrc4e
             lda #4
             sta cb0_1
             lda #8
             sta cb1_0
             lda #0ch
             sta cb1_1
             jmp vrc_ready

not_vrc4e:   jmp done		;wasn't a valid version, exit

vrc_ready:   lda #1
             sta romsiz
             ldy #004h
             jsr comp_bank
             beq gotit
             asl romsiz
             ldy #008h
             jsr comp_bank
             beq gotit
             asl romsiz
             ldy #010h
             jsr comp_bank
             beq gotit
             asl romsiz
             ldy #020h
             jsr comp_bank
             beq gotit
             asl romsiz

gotit:       lsr romsiz
             lda #0
             ror a
             sta temp3_hi
             jsr send_byte
             lda romsiz
             jsr send_byte
             lda romsiz
             asl a
             asl a
             asl a
             sta temp3_lo    ;# 8K banks
             lda #01h
             jsr send_byte

send_plp:    lda temp3_hi
             sta 08000h
             inc temp3_hi
             lda #0
             sta temp1_lo
             lda #080h
             sta temp1_hi
             ldx #020h
             ldy #0

send_plp2:   lda (temp1),y
             jsr send_byte
             iny
             bne send_plp2
             inc temp1_hi
             jsr baton
             dex
             bne send_plp2    ;send 8K bank
             dec temp3_lo
             bne send_plp
             

;read CHR stuff             
             
             
no_ram3:     lda #1
             sta romsiz
             ldy #020h
             jsr comp_bank2
             beq gotit2
             asl romsiz
             ldy #040h
             jsr comp_bank2
             beq gotit2
             asl romsiz
             ldy #080h
             jsr comp_bank2
             beq gotit2
             asl romsiz
             
gotit2:      lda romsiz
             pha
             lsr romsiz
             lda #0
             sta temp3_hi    ;start out at 0
             ror a
             jsr send_byte
             lda romsiz
             jsr send_byte
             lda #02h
             jsr send_byte
             pla
             asl a
             asl a
             asl a
             asl a
             asl a  ;1,2,4,8 == 20,40,80,00
             sta temp3_lo

send_plp3:   lda temp3_hi
             ldx vrc_version
             cpx #2ah
             bne noshift
             asl a

noshift:     sta 0b000h
             lsr a
             lsr a
             lsr a
             lsr a
             ldx cb0_1
             sta (0b000h),x      ;current bank
             lda #00h
             jsr wr_ppu
             lda 2007h      ;set up PPU
             ldy #0
             ldx #4

send_plp4:   lda 2007h
             jsr send_byte
             iny
             bne send_plp4
             jsr baton
             dex
             bne send_plp4
             inc temp3_hi
             dec temp3_lo
             bne send_plp3
             
;check for save-game RAM and back it up if it exists             

no_chr:      lda #001h
             sta 09002h
             
             jsr chk_wram
             bne done
             lda #020h
             tax
             jsr send_byte
             lda #0
             jsr send_byte
             lda #3
             jsr send_byte
             lda #0
             sta temp1_lo
             tay
             lda #060h
             sta temp1_hi
            
sr_lp:       lda (temp1),y
             jsr send_byte
             iny
             bne sr_lp
             inc temp1_hi
             jsr baton
             dex
             bne sr_lp
             
             lda #000h
             sta 09002h

done:        lda #0
             jsr send_byte
             lda #0
             jsr send_byte
             lda #0
             jsr send_byte

             rts


;y = bank to compare
;z=same
comp_bank:   ldx #0
             stx temp3_lo    ;lower pointer
             sty temp3_hi    ;upper pointer
             sty temp4       ;# blocks to compare

cb_loop:     lda #000h       ;init pointers
             sta temp1_lo
             sta temp2_lo
             lda #080h
             sta temp1_hi
             lda #0a0h
             sta temp2_hi    ;pointers 1,2 to 8000/a000
             
             lda temp3_lo
             sta 08000h
             lda temp3_hi
             sta 0a000h      ;write in current banks
             
             ldx #020h
             ldy #0

cb_loop2:    lda (temp1),y
             cmp (temp2),y
             bne diff
             iny
             bne cb_loop2
             inc temp1_hi
             inc temp2_hi
             dex
             bne cb_loop2
             inc temp3_lo
             inc temp3_hi
             dec temp4
             bne cb_loop

diff:        rts

;y = bank to compare
;z=same
;for CHR
comp_bank2:  ldx #0
             stx temp3_lo    ;lower pointer
             sty temp3_hi    ;upper pointer
             sty temp4       ;# blocks to compare

cc_loop:     lda temp3_lo
             ldx vrc_version
             cpx #2ah
             bne noshift2
             asl a

noshift2:    sta 0b000h
             lsr a
             lsr a
             lsr a
             lsr a
             ldx cb0_1
             sta (0b000h),x

             lda temp3_hi
             ldx vrc_version
             cpx #2ah
             bne noshift3
             asl a

noshift3:    ldx cb1_0
             sta (0b000h),x      ;write in current banks
             lsr a
             lsr a
             lsr a
             lsr a
             ldx cb1_1
             sta (0b000h),x

             ldx #004h
             lda #000h
             sta curr_bank   ;reset current bank

cc_loop2:    ldy #0
             lda curr_bank
             sta 2006h       ;pointer =000h
             sty 2006h
             lda 2007h       ;garbage read

ql:          lda 2007h
             sta 0300h,y
             iny
             bne ql          ;load 256 bytes for testing
             lda curr_bank
             clc
             adc #4
             inc curr_bank
             sta 2006h       ;pointer =400h
             sty 2006h
             lda 2007h       ;garbage read

cc_loop3:    lda 2007h
             cmp 0300h,y
             bne diff2
             iny
             bne cc_loop3
             dex
             bne cc_loop2
             inc temp3_lo
             inc temp3_hi
             dec temp4
             bne cc_loop

diff2:       rts


             .fill 07c0h-*,0ffh   ;fill rest to get 1K of data

             .db "VRC 2ab,4a-e"
             .fill 07ceh-*,0
             .fill 07d0h-*,0ffh

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
